@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2010 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
 // RTP sink for AC3 audio
 // Implementation
 
@@ -24,7 +24,8 @@ AC3AudioRTPSink::AC3AudioRTPSink(UsageEnvironment& env, Groupsock* RTPgs,
 				 u_int8_t rtpPayloadFormat,
 				 u_int32_t rtpTimestampFrequency)
   : AudioRTPSink(env, RTPgs, rtpPayloadFormat,
-		       rtpTimestampFrequency, "AC3") {
+		       rtpTimestampFrequency, "AC3"),
+    fTotNumFragmentsUsed(0) {
 }
 
 AC3AudioRTPSink::~AC3AudioRTPSink() {
@@ -49,23 +50,31 @@ void AC3AudioRTPSink
 ::doSpecialFrameHandling(unsigned fragmentationOffset,
 			 unsigned char* frameStart,
 			 unsigned numBytesInFrame,
-			 struct timeval frameTimestamp,
+			 struct timeval framePresentationTime,
 			 unsigned numRemainingBytes) {
-  // Update the "NDU" header.
-  // Also set the "Data Unit Header" for the frame, because we
-  // have already allotted space for this, by virtue of the fact that
-  // (for now) we pack only one frame in each RTP packet:
+  // Set the 2-byte "payload header", as defined in RFC 4184.
   unsigned char headers[2];
-  headers[0] = numFramesUsedSoFar() + 1;
 
   Boolean isFragment = numRemainingBytes > 0 || fragmentationOffset > 0;
-  unsigned const totalFrameSize
-    = fragmentationOffset + numBytesInFrame + numRemainingBytes;
-  unsigned const fiveEighthsPoint = totalFrameSize/2 + totalFrameSize/8;
-  Boolean haveFiveEighths
-    = fragmentationOffset == 0 && numBytesInFrame >= fiveEighthsPoint;
-  headers[1] = (isFragment<<5)|(haveFiveEighths<<4); // F|B
-      // Note: TYP==0, RDT==0 ???, T==0 ???
+  if (!isFragment) {
+    headers[0] = 0; // One or more complete frames
+    headers[1] = 1; // because we (for now) allow at most 1 frame per packet
+  } else {
+    if (fragmentationOffset > 0) {
+      headers[0] = 3; // Fragment of frame other than initial fragment
+    } else {
+      // An initial fragment of the frame
+      unsigned const totalFrameSize = fragmentationOffset + numBytesInFrame + numRemainingBytes;
+      unsigned const fiveEighthsPoint = totalFrameSize/2 + totalFrameSize/8;
+      headers[0] = numBytesInFrame >= fiveEighthsPoint ? 1 : 2;
+
+      // Because this outgoing packet will be full (because it's an initial fragment), we can compute how many total
+      // fragments (and thus packets) will make up the complete AC-3 frame:
+      fTotNumFragmentsUsed = (totalFrameSize + (numBytesInFrame-1))/numBytesInFrame;
+    }
+
+    headers[1] = fTotNumFragmentsUsed;
+  }
 
   setSpecialHeaderBytes(headers, sizeof headers);
 
@@ -79,14 +88,10 @@ void AC3AudioRTPSink
   // to set the packet's timestamp:
   MultiFramedRTPSink::doSpecialFrameHandling(fragmentationOffset,
 					     frameStart, numBytesInFrame,
-					     frameTimestamp,
+					     framePresentationTime,
 					     numRemainingBytes);
 }
 
 unsigned AC3AudioRTPSink::specialHeaderSize() const {
-  // There's a 1 byte "NDU" header.
-  // There's also a 1-byte "Data Unit Header" preceding each frame in
-  // the RTP packet, but since we (for now) pack only one frame in
-  // each RTP packet, we also count this here:
-  return 1 + 1;
+  return 2;
 }
