@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2010 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
 // RTP sink for H.264 video (RFC 3984)
 // Implementation
 
@@ -32,9 +32,9 @@ H264VideoRTPSink
 }
 
 H264VideoRTPSink::~H264VideoRTPSink() {
+  fSource = fOurFragmenter; // hack: in case "fSource" had gotten set to NULL before we were called
   delete[] fFmtpSDPLine;
-  Medium::close(fOurFragmenter);
-  fSource = NULL;
+  stopPlaying();
 }
 
 H264VideoRTPSink*
@@ -62,19 +62,17 @@ Boolean H264VideoRTPSink::continuePlaying() {
 }
 
 void H264VideoRTPSink::stopPlaying() {
-  // First, call the parent class's implementation, to stop our fragmenter object
-  // (and its source):
+  // First, call the parent class's implementation, to stop our fragmenter object (and its source):
   MultiFramedRTPSink::stopPlaying();
 
   // Then, close our 'fragmenter' object:
   Medium::close(fOurFragmenter); fOurFragmenter = NULL;
-  fSource = NULL;
 }
 
 void H264VideoRTPSink::doSpecialFrameHandling(unsigned /*fragmentationOffset*/,
 					      unsigned char* /*frameStart*/,
 					      unsigned /*numBytesInFrame*/,
-					      struct timeval frameTimestamp,
+					      struct timeval framePresentationTime,
 					      unsigned /*numRemainingBytes*/) {
   // Set the RTP 'M' (marker) bit iff
   // 1/ The most recently delivered fragment was the end of (or the only fragment of) an NAL unit, and
@@ -90,7 +88,7 @@ void H264VideoRTPSink::doSpecialFrameHandling(unsigned /*fragmentationOffset*/,
     }
   }
 
-  setTimestamp(frameTimestamp);
+  setTimestamp(framePresentationTime);
 }
 
 Boolean H264VideoRTPSink
@@ -112,7 +110,7 @@ char const* H264VideoRTPSink::auxSDPLine() {
   framerSource->getSPSandPPS(sps, spsSize, pps, ppsSize);
   if (sps == NULL || pps == NULL) return NULL; // our source isn't ready
 
-  u_int8_t profile_level_id;
+  u_int32_t profile_level_id;
   if (spsSize < 4) { // sanity check
     profile_level_id = 0;
   } else {
@@ -158,7 +156,7 @@ H264FUAFragmenter::H264FUAFragmenter(UsageEnvironment& env,
 
 H264FUAFragmenter::~H264FUAFragmenter() {
   delete[] fInputBuffer;
-  fInputSource = NULL; // so that the subsequent ~FramedFilter doesn't delete it
+  detachInputSource(); // so that the subsequent ~FramedFilter() doesn't delete it
 }
 
 void H264FUAFragmenter::doGetNextFrame() {
