@@ -541,7 +541,7 @@ unsigned RTSPClient::sendRequest(RequestRecord* request) {
     unsigned contentStrLen = strlen(contentStr);
     if (contentStrLen > 0) {
       char const* contentLengthHeaderFmt =
-	"Content-length: %d\r\n";
+	"Content-Length: %d\r\n";
       unsigned contentLengthHeaderSize = strlen(contentLengthHeaderFmt)
 	+ 20 /* max int len */;
       contentLengthHeader = new char[contentLengthHeaderSize];
@@ -811,11 +811,13 @@ void RTSPClient::handleIncomingRequest() {
   char urlPreSuffix[RTSP_PARAM_STRING_MAX];
   char urlSuffix[RTSP_PARAM_STRING_MAX];
   char cseq[RTSP_PARAM_STRING_MAX];
+  unsigned contentLength;
   if (!parseRTSPRequestString(fResponseBuffer, fResponseBytesAlreadySeen,
 			      cmdName, sizeof cmdName,
 			      urlPreSuffix, sizeof urlPreSuffix,
 			      urlSuffix, sizeof urlSuffix,
-			      cseq, sizeof cseq)) {
+			      cseq, sizeof cseq,
+			      contentLength)) {
     return;
   } else {
     if (fVerbosityLevel >= 1) {
@@ -1082,7 +1084,11 @@ Boolean RTSPClient::handleGET_PARAMETERResponse(char const* parameterName, char*
       unsigned parameterNameLen = strlen(parameterName);
       // ASSERT: parameterNameLen >= 2;
       parameterNameLen -= 2; // because of the trailing \r\n
-      if (_strncasecmp(resultValueString, parameterName, parameterNameLen) != 0) break; // parameter name wasn't in the output
+      if (_strncasecmp(resultValueString, parameterName, parameterNameLen) != 0) {
+	// The parameter name wasn't in the output, so just return an empty string:
+	resultValueString[0] = '\0';
+	return True;
+      }
       resultValueString += parameterNameLen;
       if (resultValueString[0] == ':') ++resultValueString;
       while (resultValueString[0] == ' ' || resultValueString[0] == '\t') ++resultValueString;
@@ -1465,7 +1471,6 @@ void RTSPClient::handleResponseBytes(int newBytesRead) {
       } else if (checkForHeader(lineStart, "Location:", 9, headerParamsStr)) {
         setBaseURL(headerParamsStr);
       }
-      // For now, omit parsing the "Server:" header (unless someone convinces us that we still need to treat Windows Media Server especially
     }
     if (!reachedEndOfHeaders) break; // an error occurred
 
@@ -1484,7 +1489,7 @@ void RTSPClient::handleResponseBytes(int newBytesRead) {
       unsigned remainingBufferSize = responseBufferSize - fResponseBytesAlreadySeen;
       if (numExtraBytesNeeded > remainingBufferSize) {
         char tmpBuf[200];
-	sprintf(tmpBuf, "Response buffer size (%d) is too small for \"Content-length:\" %d (need a buffer size of >= %d bytes\n",
+	sprintf(tmpBuf, "Response buffer size (%d) is too small for \"Content-Length:\" %d (need a buffer size of >= %d bytes\n",
                 responseBufferSize, contentLength, fResponseBytesAlreadySeen + numExtraBytesNeeded);
 	envir().setResultMsg(tmpBuf);
         break;
