@@ -43,7 +43,7 @@ WAVAudioFileServerMediaSubsession
 }
 
 void WAVAudioFileServerMediaSubsession
-::seekStreamSource(FramedSource* inputSource, double seekNPT, double streamDuration) {
+::seekStreamSource(FramedSource* inputSource, double& seekNPT, double streamDuration, u_int64_t& numBytes) {
   WAVAudioFileSource* wavSource;
   if (fBitsPerSample == 16) {
     // "inputSource" is a filter; its input source is the original WAV file source:
@@ -58,6 +58,7 @@ void WAVAudioFileServerMediaSubsession
 
   unsigned numDurationSamples = (unsigned)(streamDuration*fSamplingFrequency);
   unsigned numDurationBytes = (numDurationSamples*fNumChannels*fBitsPerSample)/8;
+  numBytes = (u_int64_t)numDurationBytes;
 
   wavSource->seekToPCMByte(seekByteNumber, numDurationBytes);
 }
