@@ -26,23 +26,16 @@ DynamicRTSPServer*
 DynamicRTSPServer::createNew(UsageEnvironment& env, Port ourPort,
 			     UserAuthenticationDatabase* authDatabase,
 			     unsigned reclamationTestSeconds) {
-  int ourSocket = -1;
+  int ourSocket = setUpOurSocket(env, ourPort);
+  if (ourSocket == -1) return NULL;
 
-  do {
-    int ourSocket = setUpOurSocket(env, ourPort);
-    if (ourSocket == -1) break;
-
-    return new DynamicRTSPServer(env, ourSocket, ourPort, authDatabase, reclamationTestSeconds);
-  } while (0);
-
-  if (ourSocket != -1) ::closeSocket(ourSocket);
-  return NULL;
+  return new DynamicRTSPServer(env, ourSocket, ourPort, authDatabase, reclamationTestSeconds);
 }
 
 DynamicRTSPServer::DynamicRTSPServer(UsageEnvironment& env, int ourSocket,
 				     Port ourPort,
 				     UserAuthenticationDatabase* authDatabase, unsigned reclamationTestSeconds)
-  : RTSPServer(env, ourSocket, ourPort, authDatabase, reclamationTestSeconds) {
+  : RTSPServerSupportingHTTPStreaming(env, ourSocket, ourPort, authDatabase, reclamationTestSeconds) {
 }
 
 DynamicRTSPServer::~DynamicRTSPServer() {
@@ -79,6 +72,15 @@ DynamicRTSPServer::lookupServerMediaSession(char const* streamName) {
   }
 }
 
+// Special code for handling Matroska files:
+static char newMatroskaDemuxWatchVariable;
+static MatroskaFileServerDemux* demux;
+static void onMatroskaDemuxCreation(MatroskaFileServerDemux* newDemux, void* /*clientData*/) {
+  demux = newDemux;
+  newMatroskaDemuxWatchVariable = 1;
+}
+// END Special code for handling Matroska files:
+
 #define NEW_SMS(description) do {\
 char const* descStr = description\
     ", streamed by the LIVE555 Media Server";\
@@ -165,6 +167,19 @@ static ServerMediaSession* createNewSMS(UsageEnvironment& env,
 
     NEW_SMS("DV Video");
     sms->addSubsession(DVVideoFileServerMediaSubsession::createNew(env, fileName, reuseSource));
+  } else if (strcmp(extension, ".mkv") == 0) {
+    // Assumed to be a Matroska file
+    NEW_SMS("Matroska video+audio+(optional)subtitles");
+
+    // Create a Matroska file server demultiplexor for the specified file.  (We enter the event loop to wait for this to complete.)
+    newMatroskaDemuxWatchVariable = 0;
+    MatroskaFileServerDemux::createNew(env, fileName, onMatroskaDemuxCreation, NULL);
+    env.taskScheduler().doEventLoop(&newMatroskaDemuxWatchVariable);
+
+    ServerMediaSubsession* smss;
+    while ((smss = demux->newServerMediaSubsession()) != NULL) {
+      sms->addSubsession(smss);
+    }
   }
 
   return sms;
