@@ -64,6 +64,7 @@ int main(int argc, char** argv) {
   *env << "\t\".amr\" => an AMR Audio file\n";
   *env << "\t\".dv\" => a DV Video file\n";
   *env << "\t\".m4e\" => a MPEG-4 Video Elementary Stream file\n";
+  *env << "\t\".mkv\" => a Matroska audio+video+(optional)subtitles file\n";
   *env << "\t\".mp3\" => a MPEG-1 or 2 Audio file\n";
   *env << "\t\".mpg\" => a MPEG-1 or 2 Program Stream (audio+video) file\n";
   *env << "\t\".ts\" => a MPEG Transport Stream file\n";
@@ -76,7 +77,7 @@ int main(int argc, char** argv) {
   // port numbers (8000 and 8080).
 
   if (rtspServer->setUpTunnelingOverHTTP(80) || rtspServer->setUpTunnelingOverHTTP(8000) || rtspServer->setUpTunnelingOverHTTP(8080)) {
-    *env << "(We use port " << rtspServer->httpServerPortNum() << " for optional RTSP-over-HTTP tunneling.)\n";
+    *env << "(We use port " << rtspServer->httpServerPortNum() << " for optional RTSP-over-HTTP tunneling, or for HTTP live streaming (for indexed Transport Stream files only).)\n";
   } else {
     *env << "(RTSP-over-HTTP tunneling is not available.)\n";
   }
